

App.Connection = Ember.Model.extend({
    server: attr(),
    serverForced: false,
    username: attr(),
    remotePath: attr(),
    password: attr(),
    encryptedPassword: attr(),
    type: attr(),
    name: attr(),  
    nameForced: false,
    tenant: attr(),
    port: attr(),
    encryption: attr(),
    selected: false,
    save_password: attr(),
    reconnectAtLogon: attr(),
    authentication: attr(),
    status: null,
    private_key: false,
    private_key_file: attr(),

    region: attr(),
    position: attr(),
    mountpoint: attr(),

    init: function() {

    },

    typeDidChange: function() {
        console.log(this);
        if(this.isLoaded == true) {
            this.forceName();
            if(this.get("type") == "s3") {
                if(this.get("server") == undefined || this.get("server").length == 0) {
                    this.set("server", "s3.amazonaws.com");
                    this.set("serverForced", true);
                }
            }

            else if(this.get("type") == "dreamobjects") {
                this.set("server", "objects.dreamhost.com");
                this.set("serverForced", true);
            } else if(this.get("type") == "box") {
                this.set("server", "Box.com");
                this.set("serverForced", true);
            } else if(this.get("type") == "dropbox") {
                this.set("server", "dropbox.com");
                this.set("serverForced", true);
            } else if(this.get("type") == "gdrive") {
                this.set("server", "googledrive.com");
                this.set("serverForced", true);
            } else if(this.get("type") == "cloudfiles") {
                this.set("server", "https://identity.api.rackspacecloud.com/v2.0");
                this.set("serverForced", true);
            } else {
                if(this.get("serverForced") == true) {
                    this.set("server", null);
                    this.set("serverForced", null);
                }
                
            }
        }
        
    }.observes('type').on('init'),

    authenticationMethodDidChange: function() {
        if(this.get("authentication") == "publickey") {
            expandrive.keyPrompt("key", "Select your SSH key file");
            this.set("private_key_file", expandrive.privateKeyData());
        } else {
            this.set("private_key_file", null);
        }
    }.observes('authentication'),

    serverDidChange: function() {
        this.forceName();
    }.observes("server"),

    requireValidation: function() {
        if(this.get("type") == "cloudfiles" || this.get("type") == "swift") {
            if(this.get("regionsPopulated") == true || this.get("region") != undefined) {
                return false;
            }
            if(this.get("server").indexOf("/v1.0") !== -1)
                return false;
            return true;
        }
        return false;

    }.property("type", "regionsPopulated", "regions", "server"),

    validateCredentials: function() {
        if(this.get("type") == "cloudfiles" || this.get("type") == "swift") {
            self = this;
            authEndpoint = this.get("server") + "/tokens";
            authData = {};

            if(this.get("type") == "cloudfiles") {
                authData = {
                            "auth": { "RAX-KSKEY:apiKeyCredentials": 
                                        {
                                         "username": this.get("username"),
                                         "apiKey": this.get("password")
                                        }
                                    }
                            };
            } else {
                authData = {
                            "auth": { "passwordCredentials": 
                                        {
                                         "username": this.get("username"),
                                         "password": this.get("password")
                                        },
                                        "tenantName": this.get("tenant")
                                    }
                            };
            }


            console.log("calling out");
            ad = JSON.stringify(authData);
            console.log(ad);
            $.ajax({
                url: authEndpoint,
                type: "POST",
                data: ad,
                contentType: "application/json",
                success: function(data) {
                    console.log("success");
                    serviceCatalog = data["access"]["serviceCatalog"];
                    
                    var endpointList = [];
                    var ep = [];
                    for(var i = 0; i < serviceCatalog.length; i++) {
                        if(serviceCatalog[i]["type"] == "object-store") {
                            endpointList = serviceCatalog[i]["endpoints"];
                            break;
                        }
                        
                    }  
                    console.log(endpointList);

                    knownRegions = {"DFW": "Dallas", "IAD": "Northen Virginia", "ORD" : "Chicago", "HKG" : "Hong Kong", "SYD" : "Sydney", "LON" : "London"};

                    for(var i = 0; i < endpointList.length; i++) {
                        regionName = endpointList[i]["region"];

                        if(knownRegions[regionName] != undefined) {
                            regionName = knownRegions[regionName];
                        }
                        
                        ep.push({"name" : regionName, "value" : endpointList[i]["region"]});
                    }
                    self.set("regions", ep);
                    self.set("regionsPopulated", true);

                },
                error: function(XMLHttpRequest, textStatus, errorThrown) {
                    console.log("Bad credentials");
                }
            });

        }
    },

    regions: null,

    regionsPopulated: false,

    needsTenant: function() {

        if (this.get("type") == "swift" && this.get("server").indexOf("/v2.0") !== -1) {
            return true;
        }

        return false;

    }.property("type", "server"),

    forceName: function() {
        if (this.get("name") == undefined || this.get("name").length == 0 || this.get("nameForced") == true) {
            if(this.get("type") == "s3") {
                this.set("name", "Amazon S3");
            }
            else if(this.get("type") == "dreamobjects") {
                this.set("name", "DreamObjects");
            } else if(this.get("type") == "cloudfiles") {
                this.set("name", "Cloud Files");
            } else if(this.get("type") == "swift") {
                this.set("name", "OpenStack Swift");
            }
            else if(this.get("type") == "box") {
                this.set("name", "Box.com");
            }
            else if(this.get("type") == "gdrive") {
                this.set("name", "Google Drive");
            }
            else if(this.get("type") == "dropbox") {
                this.set("name", "Dropbox");
                
            } else if(this.get("type") == "webdav") {
                var myURL = parseURL(this.get("server"));
                this.set("name", myURL.host);
            } else {
                if(this.get("server") && this.get("server").length > 0)
                    this.set("name", this.get("server"));
            }
            
            this.set("nameForced", true);
        } 

    },

    nickname: function(key, value) {
        if(arguments.length > 1 && value && value.length > 0) {
            this.set("name", value);
            this.set("nameForced", false);
        }

        return this.get("name");
    }.property("name"),

    sshAuthMethod: function(key, value) {
        if(arguments.length > 1) {
            if(value != "password")
                this.set("authentication", value);
        }

        if(this.get("authentication") != undefined)
            return this.get("authentication");

        return "password";
    }.property("type", "authentication"),

    activity: function(s) {
        var encrypted, new_token, password, pieces, self;
        self = this;
        if (s.indexOf("EXPANDRIVE_CONNECT_PROGRESS:drive_connected") !== -1) {

            if (settings.getBool("ShowDesktopNotifications")) {
                growl.postNotification("Connected", "ExpanDrive | " + this.get("name"));
            }
            setTimeout(function() {
                if (settings.getBool("RevealAfterConnect")) {
                    return self.reveal();
                }
            }, 1000);
            expandrive.log("connected");
            this.set('status', 'Connected');
            expandrive.log(this.get("status"));
        }
        if (s.indexOf("EXPANDRIVE_AUTH_PASSWORD:") !== -1) {
            if(this.get("password") != undefined && this.get("password") != "")
                expandrive.writeLine(this.id, this.get("password"));
            else {
                setTimeout((function() {
                    if (self.private_key) {
                        return expandrive.passwordPrompt(self.id, "Password for " + s.substring(s.lastIndexOf("\\") + 1) + ":");
                    } else {
                        return expandrive.passwordPrompt(self.id, "Password for " + s.split(":")[1]);
                    }
                }), 10);
            }
                
        }
        if (s.indexOf("EXPANDRIVE_AUTH_PRIVATEKEY_FILE") !== -1) {
            this.set("private_key", true);
            expandrive.writeLine(this.id, this.get('private_key_file'));
        }
        if (s.indexOf("EXPANDRIVE_AUTH_USERNAME:") !== -1) {
            self = this;
            setTimeout((function() {
                return expandrive.inputPrompt(self.id, "Please enter a username");
            }), 10);
        }
        if (s.indexOf("EXPANDRIVE_UPDATE_REFRESH_TOKEN:") !== -1) {
            new_token = s.split(":")[1];
            this.set('password', new_token);
            this.save();
        }
        if (s.indexOf("EXPANDRIVE_CHECK_HOSTKEY") !== -1) {
            expandrive.writeLine(this.id, "yes");
        }
        if (s.indexOf("EXPANDRIVE_GET_CONFIG:volicon_path") !== -1) {
            expandrive.log(JSON.stringify(this.cleanJSON()));
            expandrive.writeLine(this.id, "");
        }
        if (s.indexOf("EXPANDRIVE_CLAIM_MOUNTPOINT") !== -1) {
            second = s.substring(s.indexOf(":") + 1);
            third = second.substring(second.indexOf(":") + 1);

            this.set("mountpoint", third);

            
            this.set('status', 'Connecting...');
        }
        if (s.indexOf("EXPANDRIVE_CONNECT_FAILURE") !== -1) {
            alert("Could not connect to " + (this.get('server')) + ":\n" + (s.split(":")[1]));
            this.set('status', null);
            expandrive.ejectDrive(this.get("id"));
        }
        if (s.indexOf("EXPANDRIVE_CONNECT_PROGRESS:drive_connecting") !== -1) {

            this.set('status', 'Connecting...');
        }
        if (s.indexOf("EXPANDRIVE_RELEASE_MOUNTPOINT") !== -1 || s.indexOf("EXPANDRIVE_TASK_COMPLETED") !== -1) {
            this.set('status', null);
        }
        if (s.indexOf("STRONGSYNC:ACTIVITYGROWL") !== -1) {
            pieces = s.split(":");
            if (settings.getBool("ShowDesktopNotifications")) {
                return growl.postNotification(pieces[2], pieces[3]);
            }
        }
    },

    connect: function() {
        cleanJson = this.cleanJSON();
        cleanJson["name"] = this.get("nickname");
        expandrive.startDrive(this.id, JSON.stringify(cleanJson), this.get("password"));
    },

    eject: function() {
        expandrive.ejectDrive(this.id);
        this.set('status', null);
    },

    reveal: function() {
        expandrive.revealDrive(this.get("mountpoint"));
    },
        
    linked: function () {
        if(this.get("status") != null) {
            return true;
        }

        return false;

    }.property("status"),

    serverLabel: function() {
        if (this.get("type") == "s3") {
            return "S3 Server";
        }

        return "SSH Server";
    }.property("type"),

    serverPlaceholder: function() {
        if (this.get("type") == "webdav") {
            return "https://yourserver.com/dav";
        }
        if (this.get("type") == "s3") {
            return "s3.amazonaws.com";
        }
        if (this.get("type") == "dreamobjects") {
            return "objects.dreamhost.com";
        }

        return "";
    }.property("type"),

    isOauthConnection: function() {
        if (this.get("type") == "box" || this.get("type") == "gdrive" || this.get("type") == "dropbox") {
            return true;
        }
        return false;
    }.property("type"),

    isOauthAuthorized: function() {
        if (this.get("type") == "box" || this.get("type") == "gdrive" || this.get("type") == "dropbox") {
            if(this.get("username") && this.get("username").length > 1) {
                return true;
            }
        }
        return false;
    }.property("type", "username"),

    usernameLabel: function() {
        if (this.get("type") == "s3") {
            return "Access Key ID";
        }
        if (this.get("type") == "box" || this.get("type") == "gdrive" || this.get("type") == "dropbox") {
            return "Account";
        }
        return "Username";
    }.property("type"),

    passwordLabel: function() {
        if (this.get("type") == "s3") {
            return "Secret Access Key";
        }

        if (this.get("type") == "cloudfiles" || this.get("type") == "swift") {
            return "API Key";
        }

        return "Password";
    }.property("type"),

    passwordPlaceholder: function() {
        if (this.get("type") == "s3") {
            return "";
        }
        return "leave blank to use keys";
    }.property("type"),

    remotePathLabel: function() {
        if (this.get("type") == "s3") {
            return "Bucket";
        }
        return "Remote Path";
    }.property("type"),

    remotePathPlaceholder: function() {
        if (this.get("type") == "s3") {
            return "";
        }
        return "optional"
    }.property("type"),

    portPlaceholder: function() {
        if (this.get("type") == "sftp") {
            return "22";
        }

        if (this.get("type") == "ftp") {
            return "21";
        }
        return "";
    }.property("type"),

    isEncryptionShown: function() {
        if (this.get("type") == "ftp") {
            return true;
        }
        return false;
    }.property("type"),


    isPortShown: function() {
        if (this.get("type") == "ftp" || this.get("type") == "sftp") {
            return true;
        }
        return false;
    }.property("type"),

    isAuthenticationShown: function() {
        if (this.get("type") == "sftp") {
            return true;
        }
        return false;
    }.property("type"),

    isUpdated: function() {
        expandrive.log()
        if (this.get("isDirty")) {
            return true;
        }
        return false;
    }.property("server"),

    isConnected: function() {
        if(this.get("status") == "Connected") {
            return true;
        }
        return false;
    }.property("status"),

    isConnecting: function() {
        if(this.get("status") == "Connecting...") {
            return true;
        }
        return false;
    }.property("status"),

    setConnecting: function() {
        this.set("status", "Connecting...");
    },

    isIdle: function() {
        if(this.get("status") != null)
            return true;
    }.property("status"),

    url: function() {
        var u;
        u = '';
        if (this.get('type') === "s3") {
          u = 's3.amazonaws.com';
        }
        if (this.get('type') === "dreamobjects") {
          u = 'objects.dreamhost.com';
        }
        if (this.get('type') === 'Strongspace') {
          u = this.get('username') + '.strongspace.com';
        }
        if (this.get('server')) {
          u = this.get('server');
        }
        if (this.get('type') === "cloudfiles") {
          u = 'Rackspace Cloud Files';
        }
        if (this.get('type') === "dropbox") {
          u = 'dropbox.com';
        }
        if (this.get('type') === "box") {
          u = 'box.com';
        }
        if (this.get('type') === "gdrive") {
          u = 'Google Drive (' + this.get('username') + ')';
        }
        if (this.get('remotePath')) {
          u += " - " + this.get('remotePath');
        }
        return u;
    }.property("type", "username", "server", "remotePath"),

    visibleStatus: function() {
        if(this.get("status") == null) {
            return this.get("url");
        } 

        return this.get("status");
    }.property("status", "url"),

    savePassword: function() {
        if (this.get("save_password") == true) {

            try {
            	pw = keychain.save(this.get("server"), this.get("username"), this.get("password"), this.get("type"), this.get("port"));
        		if(pw != undefined && pw.length > 30) {
        			this.set("encryptedPassword", pw);
        		}
        	}
        	catch (err) {
        		expandrive.log("Couldn't save PW");
        	}
        }
    },

    cleanJSON: function() {
        j = this.toJSON();
        j["id"] = this.getPrimaryKey();
        delete j["password"];
        return j;
    }, 

    connectionTypes: [{
        name: "SSH (SFTP)",
        value: "sftp"
    }, {
        name: "FTP",
        value: "ftp"
    }, {
        name: "Amazon S3",
        value: "s3"
    }, {
        name: "WebDAV",
        value: "webdav"
    }, {
        name: "Rackspace Cloud Files",
        value: "cloudfiles"
    }, {
        name: "OpenStack Swift Storage",
        value: "swift"
    }, {
        name: "DreamHost DreamObjects",
        value: "dreamobjects"
    }, {
        name: "Box.com",
        value: "box"
    }, {
        name: "Dropbox",
        value: "dropbox"
    }, {
        name: "Google Drive",
        value: "gdrive"
    }, ],

    connectionTypeName: function() {
        t = this.get("type");
        types = this.connectionTypes;
        for(var i = 0; i < types.length; i++) {
            item = types[i];
            if(item["value"] == t)
                return item["name"];
        }
        return "";
    }.property("type")

});

App.Connection.adapter = Ember.StrongappAdapter.create();